#!/usr/bin/python
# -*- coding: utf-8 -*-

#  Copyright © 2013  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function, division

import sys, os

from gi.repository import Gtk
from gi.repository import Gedit

from .utils import *


def test_action_open_directory_from_active__with_no_docs():
    return _test_action_open_directory_from_active()
    
def test_action_open_directory_from_active__with_new_doc():
    new_doc()
    return _test_action_open_directory_from_active()
    
def test_action_open_directory_from_active__with_file():
    open_doc('no-project')
    return _test_action_open_directory_from_active()
    
def _test_action_open_directory_from_active():
    path = get_active_path()
    get_actions().action_open_directory_from_active.activate()
    def question():
        dialog = Gtk.MessageDialog(None,
                        Gtk.DialogFlags.DESTROY_WITH_PARENT,
                        Gtk.MessageType.QUESTION, Gtk.ButtonsType.YES_NO,
                        'Filemanager open with %r?' % path)
        result = dialog.run()
        try:
            assert result != Gtk.ResponseType.YES, 'Filemanager open for non poject file %r' % path
        finally:
            dialog.destroy()
    return question
    
def test_action_open_project__empty_projects_list():
    get_actions().action_open_project.activate()
    def _test():
        assert len(get_open_projects()) == 0
        assert len(get_all_projects()) == 0
    return _test
    
def test_action_open_project():
    add_project('project')
    def _test():
        assert get_open_project_paths() == ['project'], get_open_project_paths()
        assert get_all_project_paths() == ['project'], get_all_project_paths()
        assert get_open_filenames() == ['project/file1', 'project/file2',
                                        'project/file3'], get_open_filenames()
    return [(open_file, ['project/file1']),
            (open_file, ['project/file2']),
            (open_file, ['project/file3']),
            (close_project, ['project']),
            (open_project, ['project']),
            _test]
            
def test_action_close_project():
    add_project('project')
    def _test():
        assert get_open_project_paths() == [], get_open_project_paths()
        assert get_all_project_paths() == ['project'], get_all_project_paths()
        assert get_open_filenames() == [], get_open_filenames()
    return [(open_file, ['project/file1']),
            (open_file, ['project/file2']),
            (close_project, ['project']),
            _test]
            
def test_action_open_file():
    add_project('project')
    def _test():
        assert get_open_project_paths() == ['project'], get_open_projects()
        assert get_all_project_paths() == ['project'], get_all_projects()
        assert get_open_filenames() == ['project/file1', 'project/file2'], get_open_filenames()
    return [(open_file, ['project/file1']), (open_file, ['project/file2']), _test]
    
def test_action_add_directory__empty_projects_list():
    add_project('project_empty')
    def _test():
        assert len(get_open_projects()) == 0
        assert get_all_project_paths() == ['project_empty'], get_all_project_paths()
    return _test
    
def test_action_add_directory__random():
    dirs = get_random_paths('random', 11, 3)
    os.mkdir(get_path('random'))
    for d in dirs:
        os.mkdir(get_path(d))
        add_project(d)
    def _test():
        assert len(get_open_projects()) == 0
        assert set(get_all_project_paths()) == set(dirs)
    return _test
    
def test_action_add_parent():
    add_project('project/subproject')
    def _test():
        assert len(get_open_projects()) == 0
        assert get_all_project_paths() == ['project', 'project/subproject'], get_all_project_paths()
    return [(add_parent_project, ['project/subproject']), _test]
    
def test_action_remove():
    add_project('project_empty')
    def _test():
        assert len(get_open_projects()) == 0
        assert get_all_project_paths() == []
    return [(remove_project, ['project_empty']), _test]
    
def test_action_find_projects__empty_projects_list():
    get_actions().action_find.activate()
    def _test():
        assert len(get_open_projects()) == 0
        assert len(get_all_projects()) == 0
    return _test
    
def test_action_find_projects__with_project():
    add_project('subprojects')
    def _select():
        select_first_project()
        get_actions().action_find.activate()
    def _test():
        assert len(get_open_projects()) == 0
        projects = get_all_project_paths()
        assert projects == ['subprojects', 'subprojects/prj-makefile',
                            'subprojects/prj-setup', 'subprojects/prj-setup/src']
    return [_select, _test]
    
def test_all_projects_random_action():
    dirs = get_random_paths('random', 11, 3)
    os.mkdir(get_path('random'))
    for d in dirs:
        os.mkdir(get_path(d))
    dirs = {d: [] for d in dirs}
    for dirname, files in dirs.items():
        for i in range(1, random.randrange(11)):
            filename = 'f%d' % i
            with open(get_path(dirname, filename), 'w'): pass
            files.append(filename)
    added = set()
    def _open():
        path = random.choice(list(dirs.keys()))
        open_project(path)
    def _add():
        path = random.choice(list(dirs.keys()))
        added.add(path)
        add_project(path)
    def _add_parent():
        path = random.choice(list(dirs.keys()))
        parent = os.path.dirname(path)
        if path in added:
            added.add(parent)
        add_parent_project(path)
    def _remove():
        path = random.choice(list(dirs.keys()))
        try:
            added.remove(path)
        except KeyError:
            pass
        remove_project(path)
    def _test(n):
        if n <= 0:
            return
        assert len(get_open_projects()) == 0
        paths = get_all_project_paths()
        assert set(paths) == added, print_diff(sorted(added), sorted(paths)) or ''
        return [random.choice((_open, _add, _add_parent, _remove)), (_test, [n-1])]
    return (_test, [len(dirs)])
    
    
